//
//  CameraViewController.swift
//  ConPDS OCR Demo
//
//  Created by Vlad on 2/26/18.
//  Copyright © 2018 ITX Aps. All rights reserved.
//
import UIKit
import Photos

class CameraViewController: UIViewController, UINavigationControllerDelegate {
    
    var version: String!
    var image: UIImage!
    
    @IBOutlet fileprivate var captureButton: UIButton!
    
    @IBOutlet weak var button1: UIView!
    @IBOutlet weak var button2: UIView!
    @IBOutlet weak var button3: UIView!
    
    ///Displays a preview of the video output generated by the device's cameras.
    @IBOutlet fileprivate var capturePreviewView: UIView!
    
    ///Allows the user to put the camera in photo mode.
//    @IBOutlet fileprivate var photoModeButton: UIButton!
//    @IBOutlet fileprivate var toggleCameraButton: UIButton!
//    @IBOutlet fileprivate var toggleFlashButton: UIButton!
    
    ///Allows the user to put the camera in video mode.
    @IBOutlet fileprivate var videoModeButton: UIButton!
    
    let cameraController = CameraController()
    
    override var prefersStatusBarHidden: Bool { return true }
    
    @IBAction func back(_ sender: UIButton) {
        dismiss(animated: true, completion: nil)
    }
    
    @IBAction func chooseImageAction(_ sender: Any) {
        let imagePickerController = UIImagePickerController()
        imagePickerController.delegate = self
        imagePickerController.sourceType = UIImagePickerControllerSourceType.photoLibrary
        imagePickerController.allowsEditing = false

        present(imagePickerController,
                animated: true,
                completion: nil)
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        button1.layer.cornerRadius = button1.frame.width / 2.0
        button2.layer.cornerRadius = button2.frame.width / 2.0
        button3.layer.cornerRadius = button3.frame.width / 2.0
    }
    
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        if let vc = segue.destination as? ResultViewController {
            vc.image = image
            vc.engineVersion = version
        }
    }
    
}

extension CameraViewController: UIImagePickerControllerDelegate {
    
    func imagePickerController(_ picker: UIImagePickerController, didFinishPickingMediaWithInfo info: [String : Any]) {
        picker.dismiss(animated: true) {
            if let image = info[UIImagePickerControllerOriginalImage] as? UIImage {
                self.image = image
                self.performSegue(withIdentifier: "ShowResultSegue", sender: self)
            }
        }
    }
}

extension CameraViewController {
    
    override func viewWillTransition(to size: CGSize, with coordinator: UIViewControllerTransitionCoordinator) {
        self.cameraController.previewLayer?.connection?.videoOrientation = UIDevice.current.orientation == .landscapeLeft ? .landscapeRight : .landscapeLeft
    }
    
    override func viewDidLoad() {
        
        func configureCameraController() {
            cameraController.prepare {(error) in
                if let error = error {
                    print(error)
                }
                
                try? self.cameraController.displayPreview(on: self.capturePreviewView)
            }
        }
        
        func styleCaptureButton() {
            captureButton.layer.borderColor = UIColor.black.cgColor
            captureButton.layer.borderWidth = 2
            
            captureButton.layer.cornerRadius = min(captureButton.frame.width, captureButton.frame.height) / 2
        }
        
        styleCaptureButton()
        configureCameraController()
        
    }
}

extension CameraViewController {
    @IBAction func toggleFlash(_ sender: UIButton) {
        if cameraController.flashMode == .on {
            cameraController.flashMode = .off
//            toggleFlashButton.setImage(#imageLiteral(resourceName: "Flash Off Icon"), for: .normal)
        }
            
        else {
            cameraController.flashMode = .on
//            toggleFlashButton.setImage(#imageLiteral(resourceName: "Flash On Icon"), for: .normal)
        }
    }
    
    @IBAction func switchCameras(_ sender: UIButton) {
        do {
            try cameraController.switchCameras()
        }
            
        catch {
            print(error)
        }
        
        switch cameraController.currentCameraPosition {
        case .some(.front):
            break
//            toggleCameraButton.setImage(#imageLiteral(resourceName: "Front Camera Icon"), for: .normal)
            
        case .some(.rear):
            break
//            toggleCameraButton.setImage(#imageLiteral(resourceName: "Rear Camera Icon"), for: .normal)
            
        case .none:
            return
        }
    }
    
    @IBAction func captureImage(_ sender: UIButton) {
        cameraController.captureImage {(image, error) in
            guard let oldImage = image else {
                print(error ?? "Image capture error")
                return
            }
            var newOrientation: UIImageOrientation!
            if UIDevice.current.orientation == .landscapeLeft {
                newOrientation = .up;
            } else {
                newOrientation = .down;
            }
            
//            switch oldImage.imageOrientation {
//            case .up:
//                newOrientation = .left;
//                break;
//            case .left:
//                newOrientation = .down;
//                break;
//            case .down:
//                newOrientation = .right;
//                break;
//            case .right:
//
//                break;
//                // you can also handle mirrored orientations similarly ...
//            default:
//                break
//            }
            self.image = UIImage(cgImage: oldImage.cgImage!, scale: 1.0, orientation: newOrientation)
            self.performSegue(withIdentifier: "ShowResultSegue", sender: self)
        }
    }
    
}


