//
//  MenuViewController.swift
//  ConPDS OCR Demo
//
//  Created by Vlad on 2/26/18.
//  Copyright © 2018 ITX Aps. All rights reserved.
//

import UIKit
import ConPDS

class MenuViewController: UIViewController {
    
    var image: UIImage!
    var version: String!
    @IBOutlet weak var cameraOverlayView: UIView!
    @IBOutlet weak var tableView: UITableView!
    @IBOutlet weak var currentVersionLabel: UILabel!    
    
    let engineVersions = ["3.4.0"]
    
    override func viewDidLoad() {
        super.viewDidLoad()
        version = engineVersions.first!
        let tap = UITapGestureRecognizer(target: self, action: #selector(showTableView))
        currentVersionLabel.isUserInteractionEnabled = true
        currentVersionLabel.addGestureRecognizer(tap)
        tableView.layer.cornerRadius = 5.0
        tableView.layer.opacity = 0.0
        License.shared.getLicenseInfo(apiKey: <#Enter your API key#>,
                                      licenseApiKey: <#Enter your license API key#>) { (info, error) in
            if error != nil {
                return
            }
        }
    }
    
    @IBAction func takePhoto(_ sender: UIButton) {
        performSegue(withIdentifier: "ShowResult", sender: self)
    }
    
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        if let vc = segue.destination as? CameraViewController {
            vc.image = image
            vc.version = version
        }
    }
    
    @IBAction func showTableViewButtonPressed(_ sender: UIButton) {
        showTableView()
    }

    @IBAction func learnMore(_ sender: UIButton) {
        let url = URL(string: "http://www.conpds.com/other-services/container-ocr-engine")!
        if UIApplication.shared.canOpenURL(url) {
            if #available(iOS 10,*) {
                UIApplication.shared.open(url, options: [:], completionHandler: nil)
            } else {
                UIApplication.shared.openURL(url)
            }
        }
    }
    
    @objc private func showTableView() {
        tableView.layer.opacity = 1.0
        let animation = CABasicAnimation(keyPath: "bounds")
        animation.fromValue = NSValue(cgRect: currentVersionLabel.frame)
        animation.toValue = NSValue(cgRect: tableView.frame)
        animation.timingFunction = CAMediaTimingFunction(name: kCAMediaTimingFunctionEaseIn)
        let opacity = CABasicAnimation(keyPath: "opacity")
        opacity.fromValue = 0.0
        opacity.toValue = 1.0
        let animationGroup = CAAnimationGroup()
        animationGroup.duration = 0.3
        animationGroup.animations = [animation, opacity]
        tableView.layer.add(animationGroup, forKey: nil)
    }
    
    private func hideTableView() {
        tableView.layer.opacity = 0.0
        let animation = CABasicAnimation(keyPath: "bounds")
        animation.fromValue = NSValue(cgRect: tableView.frame)
        animation.toValue = NSValue(cgRect: currentVersionLabel.frame)
        animation.timingFunction = CAMediaTimingFunction(name: kCAMediaTimingFunctionEaseIn)
        let opacity = CABasicAnimation(keyPath: "opacity")
        opacity.fromValue = 1.0
        opacity.toValue = 0.0
        let animationGroup = CAAnimationGroup()
        animationGroup.duration = 0.3
        animationGroup.animations = [animation, opacity]
        tableView.layer.add(animationGroup, forKey: nil)
    }
}


// MARK: - Table View Delegate & Data Source
extension MenuViewController: UITableViewDelegate, UITableViewDataSource {
    // Data Source
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return engineVersions.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "VersionNumberCell", for: indexPath)
        cell.textLabel?.text = engineVersions[indexPath.row]
        return cell
    }
    
    //Delegate
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        version = engineVersions[indexPath.row]
        currentVersionLabel.text = "\(version ?? "0.0.0")"
        hideTableView()
    }
    
    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        return 50
    }
}

// MARK: - Image Picker Delegate methods
extension MenuViewController: UIImagePickerControllerDelegate, UINavigationControllerDelegate {
    func imagePickerController(_ picker: UIImagePickerController, didFinishPickingMediaWithInfo info: [String : Any]) {
        image = info[UIImagePickerControllerOriginalImage] as! UIImage        
        picker.dismiss(animated: true, completion: {
            DispatchQueue.main.async { [weak self] in
                self?.performSegue(withIdentifier: "ShowResult", sender: self)
            }
        })
    }
    
    func imagePickerControllerDidCancel(_ picker: UIImagePickerController) {
        picker.dismiss(animated: true, completion: nil)
    }
    
}
