//
//  ResultViewController.swift
//  ConPDS OCR Demo
//
//  Created by Vlad on 2/26/18.
//  Copyright © 2018 ITX Aps. All rights reserved.
//

import UIKit
import ConPDS

class ResultViewController: UIViewController {
    
    @IBOutlet weak var textView: UITextView!
    @IBOutlet weak var imageView: UIImageView!
    @IBOutlet weak var progressView: UIView!
    
    var image: UIImage!
    var engineVersion: String!
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        guard let image = self.image else {
            return            
        }
        progressView.layer.opacity = 0
        recognize(image: image, engineVersion: engineVersion)
    }
    
    private func recognize(image: UIImage, engineVersion: String) {
        makeProgressViewVisible(true)
        let startExecution = Date()
        let recognizer = RecognitionWrapper()
        do {
            let codes = try recognizer.recognizeCodes(image, engineVersion: engineVersion)
            let endExecution = Date()
            let recognitionTime = endExecution.timeIntervalSince(startExecution)
            textView.text = "Recognition: Success\n\nContainer ID: \(codes[0] as! String)\n\nCheck Digit verification: \(checkDigit(codes[0] as? String))\n\nISO Size and type: \(codes[1] as! String)\n\nRecognition time: \(Int(recognitionTime * 1000))ms\n\nRaw: ocr-succes;\(codes[0] as! String)|\(codes[1] as! String);\(checkDigit(codes[0] as? String));\(Int(recognitionTime * 1000))ms"
            
        } catch {
            let endExecution = Date()
            let recognitionTime = endExecution.timeIntervalSince(startExecution)
            textView.text = "Recognition: Failure\n\nRecognition time:  \(Int(recognitionTime * 1000))ms\n\nRaw: ocr-failure;\(Int(recognitionTime * 1000))ms"
        }
        imageView.image = image
        makeProgressViewVisible(false)
    }
    
    func makeProgressViewVisible(_ visible: Bool) {
        UIView.animate(withDuration: 0.3) { [weak self] in
            self?.progressView.layer.opacity = visible ? 1.0 : 0.0
        }
    }
    
    @IBAction func back(_ sender: UIButton) {
        dismiss(animated: true, completion: nil)
    }
    
    func checkDigit(_ ocrCode: String?) -> Bool {
        let chars = ["0","1","2","3","4","5","6","7","8","9","A","?","B","C","D","E","F","G","H","I","J","K","?","L","M","N","O","P","Q","R","S","T","U","?","V","W","X","Y","Z"]
        var chars2num = [Character: Int]()
        var index = 0
        for char in chars {
            chars2num[Character(char)] = index
            index += 1
        }
        guard let ocrCode = ocrCode else {
            return false
        }
        if ocrCode.count < 11 {
            return false
        }
        let check = ocrCode.suffix(1).first!
        var x = 0
        var sum = 0
        
        let first10 = String(ocrCode[ocrCode.startIndex..<ocrCode.index(ocrCode.endIndex, offsetBy: -1)])
        for c in first10 {
            let num = chars2num[c] ?? 0
            sum += (num * (1 << x))
            x += 1
        }
        return sum % 11 % 10 == (chars2num[check] ?? 0)
    }
    
}
